@@ -24,6 +24,7 @@
  */
 
 #include <glib/gi18n-lib.h>
+#include <glib.h>
 #include <glib/gprintf.h>
 #include <libgda/gda-data-model.h>
 #include <libgda/gda-data-model-private.h>
@@ -46,6 +47,10 @@ extern gchar *gda_lang_locale;
 static GStaticRecMutex init_mutex = G_STATIC_REC_MUTEX_INIT;
 static void gda_data_model_class_init (gpointer g_class);
 
+static xmlNodePtr gda_data_model_to_xml_node (GdaDataModel *model, const gint *cols, gint nb_cols, 
+					      const gint *rows, gint nb_rows, const gchar *name);
+
+
 /* signals */
 enum {
 	CHANGED,
@@ -74,7 +79,7 @@ gda_data_model_get_type (void)
 
 	if (G_UNLIKELY (type == 0)) {
 		static const GTypeInfo info = {
-			sizeof (GdaDataModelClass),
+			sizeof (GdaDataModelIface),
 			(GBaseInitFunc) gda_data_model_class_init,
 			(GBaseFinalizeFunc) NULL,
 			(GClassInitFunc) NULL,
@@ -102,43 +107,77 @@ gda_data_model_class_init (gpointer g_class)
 
 	g_static_rec_mutex_lock (&init_mutex);
 	if (! initialized) {
+		/**
+		 * GdaDataModel::changed
+		 * @model: the #GdaDataModel
+		 *
+		 * Gets emitted when any value in @model has been changed
+		 */
 		gda_data_model_signals[CHANGED] =
 			g_signal_new ("changed",
 				      GDA_TYPE_DATA_MODEL,
 				      G_SIGNAL_RUN_LAST,
-				      G_STRUCT_OFFSET (GdaDataModelClass, changed),
+				      G_STRUCT_OFFSET (GdaDataModelIface, changed),
 				      NULL, NULL,
 				      g_cclosure_marshal_VOID__VOID,
 				      G_TYPE_NONE, 0);
+		/**
+		 * GdaDataModel::row-inserted
+		 * @model: the #GdaDataModel
+		 * @row: the row number
+		 *
+		 * Gets emitted when a row has been inserted in @model
+		 */
 		gda_data_model_signals[ROW_INSERTED] =
 			g_signal_new ("row-inserted",
 				      GDA_TYPE_DATA_MODEL,
 				      G_SIGNAL_RUN_LAST,
-				      G_STRUCT_OFFSET (GdaDataModelClass, row_inserted),
+				      G_STRUCT_OFFSET (GdaDataModelIface, row_inserted),
 				      NULL, NULL,
 				      g_cclosure_marshal_VOID__INT,
 				      G_TYPE_NONE, 1, G_TYPE_INT);
+		/**
+		 * GdaDataModel::row-updated
+		 * @model: the #GdaDataModel
+		 * @row: the row number
+		 *
+		 * Gets emitted when a row has been modified in @model
+		 */
 		gda_data_model_signals[ROW_UPDATED] =
 			g_signal_new ("row-updated",
 				      GDA_TYPE_DATA_MODEL,
 				      G_SIGNAL_RUN_LAST,
-				      G_STRUCT_OFFSET (GdaDataModelClass, row_updated),
+				      G_STRUCT_OFFSET (GdaDataModelIface, row_updated),
 				      NULL, NULL,
 				      g_cclosure_marshal_VOID__INT,
 				      G_TYPE_NONE, 1, G_TYPE_INT);
+		/**
+		 * GdaDataModel::row-removed
+		 * @model: the #GdaDataModel
+		 * @row: the row number
+		 *
+		 * Gets emitted when a row has been removed from @model
+		 */
 		gda_data_model_signals[ROW_REMOVED] =
 			g_signal_new ("row-removed",
 				      GDA_TYPE_DATA_MODEL,
 				      G_SIGNAL_RUN_LAST,
-				      G_STRUCT_OFFSET (GdaDataModelClass, row_removed),
+				      G_STRUCT_OFFSET (GdaDataModelIface, row_removed),
 				      NULL, NULL,
 				      g_cclosure_marshal_VOID__INT,
 				      G_TYPE_NONE, 1, G_TYPE_INT);
+		/**
+		 * GdaDataModel::reset
+		 * @model: the #GdaDataModel
+		 *
+		 * Gets emitted when @model's contents has been completely reset (the number and
+		 * type of columns may also have changed)
+		 */
 		gda_data_model_signals[RESET] =
 			g_signal_new ("reset",
 				      GDA_TYPE_DATA_MODEL,
 				      G_SIGNAL_RUN_LAST,
-				      G_STRUCT_OFFSET (GdaDataModelClass, reset),
+				      G_STRUCT_OFFSET (GdaDataModelIface, reset),
 				      NULL, NULL,
 				      g_cclosure_marshal_VOID__VOID,
 				      G_TYPE_NONE, 0);
@@ -540,7 +579,7 @@ gda_data_model_set_column_title (GdaDataModel *model, gint col, const gchar *tit
  * Retrieves the data stored in the given position (identified by
  * the @col and @row parameters) on a data model.
  *
- * This is the main function for accessing data in a model which allow random access to its data.
+ * This is the main function for accessing data in a model which allows random access to its data.
  * To access data in a data model using a cursor, use a #GdaDataModelIter object, obtained using
  * gda_data_model_create_iter().
  *
@@ -568,7 +607,7 @@ gda_data_model_get_value_at (GdaDataModel *model, gint col, gint row, GError **e
 	else {
 		/* method not supported */
 		g_set_error (error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_FEATURE_NON_SUPPORTED_ERROR,
-			     _("Data model does not support getting individual value"));
+			      "%s", _("Data model does not support getting individual value"));
 		return NULL;
 	}
 }
@@ -601,20 +640,23 @@ gda_data_model_get_typed_value_at (GdaDataModel *model, gint col, gint row, GTyp
 		cvalue = (GDA_DATA_MODEL_GET_CLASS (model)->i_get_value_at) (model, col, row, error);
 
 	if (cvalue) {
-		if (nullok && (G_VALUE_TYPE (cvalue) != GDA_TYPE_NULL) && (G_VALUE_TYPE (cvalue) != expected_type)) {
-			cvalue = NULL;
+		if (nullok && 
+		    (G_VALUE_TYPE (cvalue) != GDA_TYPE_NULL) && 
+		    (G_VALUE_TYPE (cvalue) != expected_type)) {
 			g_set_error (error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_VALUE_TYPE_ERROR,
-				     _("Data model returned value of invalid '%s' type"), gda_g_type_to_string (G_VALUE_TYPE (cvalue)));
+				     _("Data model returned value of invalid '%s' type"), 
+				     gda_g_type_to_string (G_VALUE_TYPE (cvalue)));
+			cvalue = NULL;
 		}
 		else if (!nullok && (G_VALUE_TYPE (cvalue) != expected_type)) {
-			cvalue = NULL;
-			if (G_VALUE_TYPE (cvalue) != GDA_TYPE_NULL)
+			if (G_VALUE_TYPE (cvalue) == GDA_TYPE_NULL)
 				g_set_error (error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_VALUE_TYPE_ERROR,
-					     _("Data model returned invalid NULL value"));
+					      "%s", _("Data model returned invalid NULL value"));
 			else
 				g_set_error (error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_VALUE_TYPE_ERROR,
 					     _("Data model returned value of invalid '%s' type"), 
 					     gda_g_type_to_string (G_VALUE_TYPE (cvalue)));
+			cvalue = NULL;
 		}
 	}
 	return cvalue;
@@ -666,7 +708,7 @@ gda_data_model_set_value_at (GdaDataModel *model, gint col, gint row, const GVal
 	else {
 		/* method not supported */
 		g_set_error (error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_FEATURE_NON_SUPPORTED_ERROR,
-			     _("Data model does not support setting individual value"));
+			      "%s", _("Data model does not support setting individual value"));
 		return FALSE;
 	}
 }
@@ -696,7 +738,7 @@ gda_data_model_set_values (GdaDataModel *model, gint row, GList *values, GError
 	else {
 		/* method not supported */
 		g_set_error (error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_FEATURE_NON_SUPPORTED_ERROR,
-			     _("Data model does not support setting values"));
+			      "%s", _("Data model does not support setting values"));
 		return FALSE;
 	}
 }
@@ -708,12 +750,18 @@ gda_data_model_set_values (GdaDataModel *model, gint row, GList *values, GError
  * Creates a new iterator object #GdaDataModelIter object which can be used to iterate through
  * rows in @model.
  *
- * The row the returned #GdaDataModelIter represents is undefined. For models which can be accessed 
- * randomly the correspoding row can be set using gda_data_model_iter_move_at_row(), 
- * and for models which are accessible sequentially only then the first row will be
- * fetched using gda_data_model_iter_move_next().
+ * Depending on the data model's implementation, a new #GdaDataModelIter object may be created,
+ * or a reference to an already existing #GdaDataModelIter may be returned.
  *
- * Returns: a new #GdaDataModelIter object, or %NULL if an error occurred
+ * If a new #GdaDataModelIter is created, then the row it represents is undefined.
+ *
+ * For models which can be accessed 
+ * randomly, any row can be set using gda_data_model_iter_move_to_row(), 
+ * and for models which are accessible sequentially only then use
+ * gda_data_model_iter_move_next() (and gda_data_model_iter_move_prev() if
+ * supported).
+ *
+ * Returns: a #GdaDataModelIter object, or %NULL if an error occurred
  */
 GdaDataModelIter *
 gda_data_model_create_iter (GdaDataModel *model)
@@ -750,7 +798,7 @@ gda_data_model_append_values (GdaDataModel *model, const GList *values, GError *
 	else {
 		/* method not supported */
 		g_set_error (error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_FEATURE_NON_SUPPORTED_ERROR,
-			     _("Data model does not support row append"));
+			      "%s", _("Data model does not support row append"));
 		return -1;
 	}
 }
@@ -772,7 +820,7 @@ gda_data_model_append_row (GdaDataModel *model, GError **error)
 
 	if (! (gda_data_model_get_access_flags (model) & GDA_DATA_MODEL_ACCESS_INSERT)) {
 		g_set_error (error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_ACCESS_ERROR,
-			     _("Model does not allow row insertion"));
+			      "%s", _("Model does not allow row insertion"));
 		return -1;
 	}
 
@@ -781,7 +829,7 @@ gda_data_model_append_row (GdaDataModel *model, GError **error)
 	else {
 		/* method not supported */
 		g_set_error (error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_FEATURE_NON_SUPPORTED_ERROR,
-			     _("Data model does not support row append"));
+			      "%s", _("Data model does not support row append"));
 		return -1;
 	}
 }
@@ -804,7 +852,7 @@ gda_data_model_remove_row (GdaDataModel *model, gint row, GError **error)
 
 	if (! (gda_data_model_get_access_flags (model) & GDA_DATA_MODEL_ACCESS_DELETE)) {
 		g_set_error (error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_ACCESS_ERROR,
-			     _("Model does not allow row deletion"));
+			      "%s", _("Model does not allow row deletion"));
 		return FALSE;
 	}
 
@@ -813,7 +861,7 @@ gda_data_model_remove_row (GdaDataModel *model, gint row, GError **error)
 	else {
 		/* method not supported */
 		g_set_error (error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_FEATURE_NON_SUPPORTED_ERROR,
-			     _("Data model does not support row removal"));
+			      "%s", _("Data model does not support row removal"));
 		return FALSE;
 	}
 }
@@ -1183,7 +1231,7 @@ xml_set_boolean (xmlNodePtr node, const gchar *name, gboolean value)
 	xmlSetProp (node, (xmlChar*)name, value ? (xmlChar*)"TRUE" : (xmlChar*)"FALSE");
 }
 
-/**
+/*
  * gda_data_model_to_xml_node
  * @model: a #GdaDataModel object.
  * @cols: an array containing which columns of @model will be exported, or %NULL for all columns
@@ -1196,7 +1244,7 @@ xml_set_boolean (xmlNodePtr node, const gchar *name, gboolean value)
  *
  * Returns: a xmlNodePtr representing the whole data model, or %NULL if an error occurred
  */
-xmlNodePtr
+static xmlNodePtr
 gda_data_model_to_xml_node (GdaDataModel *model, const gint *cols, gint nb_cols, 
 			    const gint *rows, gint nb_rows, const gchar *name)
 {
@@ -1369,7 +1417,7 @@ add_xml_row (GdaDataModel *model, xmlNodePtr xml_row, GError **error)
 		if ((gdatype == G_TYPE_INVALID) ||
 		    (gdatype == GDA_TYPE_NULL)) {
 			g_set_error (error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_XML_FORMAT_ERROR,
-				     _("Cannot retrieve column data type (type is UNKNOWN or not specified)"));
+				      "%s", _("Cannot retrieve column data type (type is UNKNOWN or not specified)"));
 			retval = FALSE;
 			break;
 		}
@@ -1432,6 +1480,7 @@ add_xml_row (GdaDataModel *model, xmlNodePtr xml_row, GError **error)
  * gda_data_model_add_data_from_xml_node
  * @model: a #GdaDataModel.
  * @node: an XML node representing a &lt;gda_array_data&gt; XML node.
+ * @error: a place to store errors, or %NULL
  *
  * Adds the data from an XML node to the given data model (see the DTD for that node
  * in the $prefix/share/libgda/dtd/libgda-array.dtd file).
@@ -1511,7 +1560,7 @@ gda_data_model_import_from_model (GdaDataModel *to, GdaDataModel *from,
 	from_iter = gda_data_model_create_iter (from);
 	if (!from_iter) {
 		g_set_error (error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_ACCESS_ERROR,
-			     _("Could not get an iterator for source data model"));
+			      "%s", _("Could not get an iterator for source data model"));
 		return FALSE;
 	}
 
@@ -1814,7 +1863,8 @@ gda_data_model_dump (GdaDataModel *model, FILE *to_stream)
 	GError *error = NULL;
 
 	g_return_if_fail (GDA_IS_DATA_MODEL (model));
-	g_return_if_fail (to_stream);
+	if (!to_stream)
+		to_stream = stdout;
 
 	if (getenv ("GDA_DATA_MODEL_DUMP_ATTRIBUTES")) 
 		dump_attrs = TRUE;
@@ -1910,11 +1960,12 @@ static gchar *
 real_gda_data_model_dump_as_string (GdaDataModel *model, gboolean dump_attributes, 
 				    gboolean dump_rows, gboolean dump_title, gboolean null_as_empty, GError **error)
 {
+#define ERROR_STRING "####"
 #define MULTI_LINE_NO_SEPARATOR
 
 	gboolean allok = TRUE;
 	GString *string;
-	gchar *offstr, *str;
+	gchar *str;
 	gint n_cols, n_rows;
 	gint *cols_size;
 	gboolean *cols_is_num;
@@ -1929,13 +1980,10 @@ real_gda_data_model_dump_as_string (GdaDataModel *model, gboolean dump_attribute
 
 	gint offset = 0;
 	gint col_offset = dump_rows ? 1 : 0;
+	GdaDataModel *ramodel = NULL;
 
 	string = g_string_new ("");
 
-        /* string for the offset */
-        offstr = g_new0 (gchar, offset+1);
-	memset (offstr, ' ', offset);
-
 	/* compute the columns widths: using column titles... */
 	n_cols = gda_data_model_get_n_columns (model);
 	n_rows = gda_data_model_get_n_rows (model);
@@ -1992,23 +2040,24 @@ real_gda_data_model_dump_as_string (GdaDataModel *model, gboolean dump_attribute
 	for (j = 0; j < n_rows; j++) {
 		for (i = 0; i < n_cols; i++) {
 			if (! dump_attributes) {
-				value = gda_data_model_get_value_at (model, i, j, error);
+				value = gda_data_model_get_value_at (model, i, j, NULL);
 				if (!value) {
-					allok = FALSE;
-					break;
+					cols_size [i + col_offset] = MAX (cols_size [i + col_offset], strlen (ERROR_STRING));
 				}
-				str = NULL;
-				if (null_as_empty) {
-					if (!value || gda_value_is_null (value))
-						str = g_strdup ("");
-				}
-				if (!str)
-					str = value ? gda_value_stringify ((GValue*)value) : g_strdup ("_null_");
-				if (str) {
-					gint w;
-					string_get_dimensions (str, &w, NULL);
-					cols_size [i + col_offset] = MAX (cols_size [i + col_offset], w);
-					g_free (str);
+				else {
+					str = NULL;
+					if (null_as_empty) {
+						if (!value || gda_value_is_null (value))
+							str = g_strdup ("");
+					}
+					if (!str)
+						str = value ? gda_value_stringify ((GValue*)value) : g_strdup ("_null_");
+					if (str) {
+						gint w;
+						string_get_dimensions (str, &w, NULL);
+						cols_size [i + col_offset] = MAX (cols_size [i + col_offset], w);
+						g_free (str);
+					}
 				}
 			}
 			else {
@@ -2021,12 +2070,10 @@ real_gda_data_model_dump_as_string (GdaDataModel *model, gboolean dump_attribute
 				g_free (str);
 			}
 		}
-		if (!allok)
-			break;
 	}
 	
 	/* actual dumping of the contents: title */
-	if (allok && dump_title) {
+	if (dump_title) {
 		const gchar *title;
 		title = g_object_get_data (G_OBJECT (model), "name");
 		if (title) {
@@ -2046,10 +2093,10 @@ real_gda_data_model_dump_as_string (GdaDataModel *model, gboolean dump_attribute
 	}
 
 	/* ...column titles...*/
-	if (allok && dump_rows) 
+	if (dump_rows) 
 		g_string_append_printf (string, "%*s", cols_size [0], "#row");
 
-	for (i = 0; (i < n_cols) && allok; i++) {
+	for (i = 0; i < n_cols; i++) {
 		gint j, max;
 		str = (gchar *) gda_data_model_get_column_title (model, i);
 		if (dump_rows || (i != 0))
@@ -2069,7 +2116,7 @@ real_gda_data_model_dump_as_string (GdaDataModel *model, gboolean dump_attribute
 	g_string_append_c (string, '\n');
 		
 	/* ... separation line ... */
-	for (i = 0; (i < n_cols + col_offset) && allok; i++) {
+	for (i = 0; i < n_cols + col_offset; i++) {
 		if (i != 0)
 			g_string_append_printf (string, "%s", sep_row);
 		for (j = 0; j < cols_size [i]; j++)
@@ -2078,38 +2125,37 @@ real_gda_data_model_dump_as_string (GdaDataModel *model, gboolean dump_attribute
 	g_string_append_c (string, '\n');
 
 	/* ... and data */
-	GdaDataModel *ramodel;
 	if (gda_data_model_get_access_flags (model) & GDA_DATA_MODEL_ACCESS_RANDOM)
 		ramodel = g_object_ref (model);
 	else {
 		if (! (gda_data_model_get_access_flags (model) & GDA_DATA_MODEL_ACCESS_CURSOR_BACKWARD)) {
 			g_set_error (error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_ACCESS_ERROR,
-				     _("Data model does not support backward cursor move, not displaying data"));
+				      "%s", _("Data model does not support backward cursor move, not displaying data"));
 			allok = FALSE;
+			goto out;
 		}
 		ramodel = gda_data_access_wrapper_new (model);
 	}
-	if (allok) {
-		for (j = 0; j < n_rows; j++) {
-			/* determine height for each column in that row */
-			gint *cols_height = g_new (gint, n_cols + col_offset);
-			gchar ***cols_str = g_new (gchar **, n_cols + col_offset);
-			gint k, kmax = 1;
 
-			if (dump_rows) {
-				str = g_strdup_printf ("%d", j);
-				cols_str [0] = g_strsplit (str, "\n", -1);
-				cols_height [0] = 1;
-				kmax = 1;
-			}
-
-			for (i = 0; i < n_cols; i++) {
-				if (!dump_attributes) {
-					value = gda_data_model_get_value_at (ramodel, i, j, error);
-					if (!value) {
-						allok = FALSE;
-						break;
-					}
+	for (j = 0; j < n_rows; j++) {
+		/* determine height for each column in that row */
+		gint *cols_height = g_new (gint, n_cols + col_offset);
+		gchar ***cols_str = g_new (gchar **, n_cols + col_offset);
+		gint k, kmax = 1;
+		
+		if (dump_rows) {
+			str = g_strdup_printf ("%d", j);
+			cols_str [0] = g_strsplit (str, "\n", -1);
+			cols_height [0] = 1;
+			kmax = 1;
+		}
+		
+		for (i = 0; i < n_cols; i++) {
+			if (!dump_attributes) {
+				value = gda_data_model_get_value_at (ramodel, i, j, NULL);
+				if (!value)
+					str = g_strdup (ERROR_STRING);
+				else {
 					str = NULL;
 					if (null_as_empty) {
 						if (!value || gda_value_is_null (value))
@@ -2118,101 +2164,99 @@ real_gda_data_model_dump_as_string (GdaDataModel *model, gboolean dump_attribute
 					if (!str)
 						str = value ? gda_value_stringify ((GValue *)value) : g_strdup ("_null_");
 				}
-				else {
-					GdaValueAttribute attrs;
-					attrs = gda_data_model_get_attributes_at (ramodel, i, j);
-					str = g_strdup_printf ("%u", attrs);
-				}
-				if (str) {
-					cols_str [i + col_offset] = g_strsplit (str, "\n", -1);
-					g_free (str);
-					cols_height [i + col_offset] = g_strv_length (cols_str [i + col_offset]);
-					kmax = MAX (kmax, cols_height [i + col_offset]);
-				}
-				else {
-					cols_str [i + col_offset] = NULL;
-					cols_height [i + col_offset] = 0;
-				}
 			}
-			if (!allok)
-				break;
-
-			for (k = 0; k < kmax; k++) {
-				for (i = 0; i < n_cols + col_offset; i++) {
-					gboolean align_center = FALSE;
-					if (i != 0) {
+			else {
+				GdaValueAttribute attrs;
+				attrs = gda_data_model_get_attributes_at (ramodel, i, j);
+				str = g_strdup_printf ("%u", attrs);
+			}
+			if (str) {
+				cols_str [i + col_offset] = g_strsplit (str, "\n", -1);
+				g_free (str);
+				cols_height [i + col_offset] = g_strv_length (cols_str [i + col_offset]);
+				kmax = MAX (kmax, cols_height [i + col_offset]);
+			}
+			else {
+				cols_str [i + col_offset] = NULL;
+				cols_height [i + col_offset] = 0;
+			}
+		}
+		
+		for (k = 0; k < kmax; k++) {
+			for (i = 0; i < n_cols + col_offset; i++) {
+				gboolean align_center = FALSE;
+				if (i != 0) {
 #ifdef MULTI_LINE_NO_SEPARATOR
-						if (k != 0)
-							g_string_append_printf (string, "%s", sep_col_e);
-						else
+					if (k != 0)
+						g_string_append_printf (string, "%s", sep_col_e);
+					else
 #endif
-							g_string_append_printf (string, "%s", sep_col);
-					}
-					if (k < cols_height [i]) 
-						str = (cols_str[i])[k];
-					else {
+						g_string_append_printf (string, "%s", sep_col);
+				}
+				if (k < cols_height [i]) 
+					str = (cols_str[i])[k];
+				else {
 #ifdef MULTI_LINE_NO_SEPARATOR
-						str = "";
+					str = "";
 #else
-						if (cols_height [i] == 0)
-							str = "";
-						else
-							str = ".";
-						align_center = TRUE;
+					if (cols_height [i] == 0)
+						str = "";
+					else
+						str = ".";
+					align_center = TRUE;
 #endif
-					}
+				}
+				
+				if (cols_is_num [i])
+					g_string_append_printf (string, "%*s", cols_size [i], str);
+				else {
+					gint j, max;
+					if (str) 
+						max = cols_size [i] - g_utf8_strlen (str, -1);
+					else
+						max = cols_size [i];
 					
-					if (cols_is_num [i])
-						g_string_append_printf (string, "%*s", cols_size [i], str);
-					else {
-						gint j, max;
-						if (str) 
-							max = cols_size [i] - g_utf8_strlen (str, -1);
-						else
-							max = cols_size [i];
-
-						if (!align_center) {
-							g_string_append_printf (string, "%s", str);
-							for (j = 0; j < max; j++)
-								g_string_append_c (string, ' ');
-						}
-						else {
-							for (j = 0; j < max / 2; j++)
-								g_string_append_c (string, ' ');
-							g_string_append_printf (string, "%s", str);
-							for ( j+= g_utf8_strlen (str, -1); j < cols_size [i]; j++)
-								g_string_append_c (string, ' ');
-						}
-						/*
-						gint j, max;
-						if (str) {
-							g_string_append_printf (string, "%s", str);
-							max = cols_size [i] - g_utf8_strlen (str, -1);
-						}
-						else
-							max = cols_size [i];
+					if (!align_center) {
+						g_string_append_printf (string, "%s", str);
 						for (j = 0; j < max; j++)
 							g_string_append_c (string, ' ');
-						*/
 					}
+					else {
+						for (j = 0; j < max / 2; j++)
+							g_string_append_c (string, ' ');
+						g_string_append_printf (string, "%s", str);
+						for ( j+= g_utf8_strlen (str, -1); j < cols_size [i]; j++)
+							g_string_append_c (string, ' ');
+					}
+					/*
+					  gint j, max;
+					  if (str) {
+					  g_string_append_printf (string, "%s", str);
+					  max = cols_size [i] - g_utf8_strlen (str, -1);
+					  }
+					  else
+					  max = cols_size [i];
+					  for (j = 0; j < max; j++)
+					  g_string_append_c (string, ' ');
+					*/
 				}
-				g_string_append_c (string, '\n');
 			}
-
-			g_free (cols_height);
-			for (i = 0; i < n_cols; i++) 
-				g_strfreev (cols_str [i]);
-			g_free (cols_str);
+			g_string_append_c (string, '\n');
 		}
-		g_string_append_printf (string, ngettext("(%d row)\n", "(%d rows)\n", n_rows), n_rows);
+		
+		g_free (cols_height);
+		for (i = 0; i < n_cols; i++) 
+			g_strfreev (cols_str [i]);
+		g_free (cols_str);
 	}
+	g_string_append_printf (string, ngettext("(%d row)\n", "(%d rows)\n", n_rows), n_rows);
 
-	g_object_unref (ramodel);
+ out:
+	if (ramodel)
+		g_object_unref (ramodel);
 	g_free (cols_size);
 	g_free (cols_is_num);
 
-	g_free (offstr);
-	
 	if (allok) {
 		str = string->str;
 		g_string_free (string, FALSE);
