@@ -22,6 +22,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#undef GDA_DISABLE_DEPRECATED
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
@@ -30,6 +31,7 @@
 #include <glib/gi18n-lib.h>
 #include <libgda/gda-log.h>
 #include <libgda/gda-util.h>
+#include <libgda/gda-server-provider.h>
 #include <libgda/gda-column.h>
 #ifdef HAVE_LOCALE_H
 #include <locale.h>
@@ -39,9 +41,19 @@
 
 #include <libgda/binreloc/gda-binreloc.h>
 
+/* we don't want to duplicate the symbols in <libgda/sqlite/keywords_hash.h>, so simply
+ * declare them as external
+ */
+extern const unsigned char UpperToLower[];
+#define charMap(X) UpperToLower[(unsigned char)(X)]
+extern int casecmp(const char *zLeft, const char *zRight, int N);
+#include "keywords_hash.c" /* this one is dynamically generated */
+
 extern gchar *gda_lang_locale;
 extern GdaAttributesManager *gda_holder_attributes_manager;
 
+#define PROV_CLASS(provider) (GDA_SERVER_PROVIDER_CLASS (G_OBJECT_GET_CLASS (provider)))
+
 /**
  * gda_g_type_to_string
  * @type: Type to convert from.
@@ -73,6 +85,10 @@ gda_g_type_to_string (GType type)
 		return "timestamp";
 	else if (type == G_TYPE_BOOLEAN)
 		return "boolean";
+	else if (type == GDA_TYPE_BLOB)
+		return "blob";
+	else if (type == GDA_TYPE_BINARY)
+		return "binary";
 	else 
 		return g_type_name (type);
 }
@@ -92,10 +108,12 @@ gda_g_type_to_string (GType type)
  *   <listitem><para>"time" for GDA_TYPE_TIME</para></listitem>
  *   <listitem><para>"timestamp" for GDA_TYPE_TIMESTAMP</para></listitem>
  *   <listitem><para>"boolean" for G_TYPE_BOOLEAN</para></listitem>
+ *   <listitem><para>"blob" for GDA_TYPE_BLOB</para></listitem>
+ *   <listitem><para>"binary" for GDA_TYPE_BINARY</para></listitem>
  *   <listitem><para>"null" for GDA_TYPE_NULL</para></listitem>
  * </itemizedlist>
  *
- * Returns: the #GType represented by the given @str.
+ * Returns: the #GType represented by the given @str, or #G_TYPE_INVALID if not found
  */
 GType
 gda_g_type_from_string (const gchar *str)
@@ -117,6 +135,10 @@ gda_g_type_from_string (const gchar *str)
 			type = GDA_TYPE_TIMESTAMP;
 		else if (!strcmp (str, "boolean"))
                         type = G_TYPE_BOOLEAN;
+		else if (!strcmp (str, "blob"))
+			type = GDA_TYPE_BLOB;
+		else if (!strcmp (str, "binary"))
+			type = GDA_TYPE_BINARY;
 		else if (!strcmp (str, "null"))
 			type = GDA_TYPE_NULL;
 		else
@@ -768,7 +790,8 @@ dml_statements_check_select_structure (GdaConnection *cnc, GdaSqlStatement *sel_
 }
 
 /**
- * gda_compute_unique_table_row_condition
+ * gda_compute_unique_table_row_condition_with_cnc
+ * @cnc: a #GdaConnection, or %NULL
  * @stsel: a #GdaSqlSelectStatement
  * @mtable: a #GdaMetaTable
  * @require_pk: set to TRUE if a primary key ir required
@@ -778,14 +801,19 @@ dml_statements_check_select_structure (GdaConnection *cnc, GdaSqlStatement *sel_
  * or DELETE statement when a row from the result of the @stsel statement has to be modified.
  *
  * Returns: a new #GdaSqlExpr, or %NULL if an error occurred.
+ *
+ * Since: 4.0.3
  */
 GdaSqlExpr*
-gda_compute_unique_table_row_condition (GdaSqlStatementSelect *stsel, GdaMetaTable *mtable, gboolean require_pk, GError **error)
+gda_compute_unique_table_row_condition_with_cnc (GdaConnection *cnc, GdaSqlStatementSelect *stsel,
+						 GdaMetaTable *mtable, gboolean require_pk, GError **error)
 {
 	gint i;
 	GdaSqlExpr *expr;
 	GdaSqlOperation *and_cond = NULL;
 
+	g_return_val_if_fail (!cnc || GDA_IS_CONNECTION (cnc), NULL);
+
 	if (mtable->pk_cols_nb == 0) {
 		g_set_error (error, GDA_SQL_ERROR, GDA_SQL_STRUCTURE_CONTENTS_ERROR,
 			     "%s", _("Table does not have any primary key"));
@@ -829,17 +857,25 @@ gda_compute_unique_table_row_condition (GdaSqlStatementSelect *stsel, GdaMetaTab
 				GdaSqlOperation *op;
 				GdaSqlExpr *opexpr;
 				GdaSqlParamSpec *pspec;
+				gchar *str;
 
 				/* equal condition */
-				op = gda_sql_operation_new (GDA_SQL_ANY_PART (and_cond ? (gpointer)and_cond : (gpointer)expr));
-				op->operator_type = GDA_SQL_OPERATOR_TYPE_EQ;
-				if (and_cond) 
-					and_cond->operands = g_slist_append (and_cond->operands, op);
-				else
+				if (and_cond) {
+					opexpr = gda_sql_expr_new (GDA_SQL_ANY_PART (and_cond));
+					op = gda_sql_operation_new (GDA_SQL_ANY_PART (opexpr));
+					opexpr->cond = op;
+					and_cond->operands = g_slist_append (and_cond->operands, opexpr);
+				}
+				else {
+					op = gda_sql_operation_new (GDA_SQL_ANY_PART (expr));
 					expr->cond = op;
+				}
+				op->operator_type = GDA_SQL_OPERATOR_TYPE_EQ;
 				/* left operand */
 				opexpr = gda_sql_expr_new (GDA_SQL_ANY_PART (op));
-				g_value_set_string (opexpr->value = gda_value_new (G_TYPE_STRING), tcol->column_name);
+				str = gda_sql_identifier_quote (tcol->column_name, cnc, NULL, FALSE, FALSE);
+				g_value_take_string (opexpr->value = gda_value_new (G_TYPE_STRING), str);
+
 				op->operands = g_slist_append (op->operands, opexpr);
 
 				/* right operand */
@@ -865,6 +901,24 @@ gda_compute_unique_table_row_condition (GdaSqlStatementSelect *stsel, GdaMetaTab
 	return NULL;
 }
 
+/**
+ * gda_compute_unique_table_row_condition
+ * @stsel: a #GdaSqlSelectStatement
+ * @mtable: a #GdaMetaTable
+ * @require_pk: set to TRUE if a primary key ir required
+ * @error: a place to store errors, or %NULL
+ * 
+ * Computes a #GdaSqlExpr expression which can be used in the WHERE clause of an UPDATE
+ * or DELETE statement when a row from the result of the @stsel statement has to be modified.
+ *
+ * Returns: a new #GdaSqlExpr, or %NULL if an error occurred.
+ */
+GdaSqlExpr*
+gda_compute_unique_table_row_condition (GdaSqlStatementSelect *stsel, GdaMetaTable *mtable, gboolean require_pk, GError **error)
+{
+	return gda_compute_unique_table_row_condition_with_cnc (NULL, stsel, mtable, require_pk, error);
+}
+
 /**
  * gda_compute_dml_statements
  * @cnc: a #GdaConnection
@@ -900,6 +954,8 @@ gda_compute_dml_statements (GdaConnection *cnc, GdaStatement *select_stmt, gbool
         GdaSqlStatement *sql_dst = NULL;
         GdaSqlStatementDelete *dst = NULL;
 
+	gchar *tmp;
+
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
 	g_return_val_if_fail (GDA_IS_STATEMENT (select_stmt), FALSE);
 	g_return_val_if_fail (gda_statement_get_statement_type (select_stmt) == GDA_SQL_STATEMENT_SELECT, FALSE);
@@ -925,14 +981,15 @@ gda_compute_dml_statements (GdaConnection *cnc, GdaStatement *select_stmt, gbool
 	stsel = (GdaSqlStatementSelect*) sel_struct->contents;
 	target = (GdaSqlSelectTarget*) stsel->from->targets->data;
 	
-	/* actual statement structure's computation */        
+	/* actual statement structure's computation */
+	tmp = gda_sql_identifier_quote (target->table_name, cnc, NULL, FALSE, FALSE);
 	if (insert_stmt) {
 		sql_ist = gda_sql_statement_new (GDA_SQL_STATEMENT_INSERT);
 		ist = (GdaSqlStatementInsert*) sql_ist->contents;
 		g_assert (GDA_SQL_ANY_PART (ist)->type == GDA_SQL_ANY_STMT_INSERT);
 
 		ist->table = gda_sql_table_new (GDA_SQL_ANY_PART (ist));
-		ist->table->table_name = g_strdup ((gchar *) target->table_name);
+		ist->table->table_name = g_strdup (tmp);
 	}
 	
 	if (update_stmt) {
@@ -941,15 +998,17 @@ gda_compute_dml_statements (GdaConnection *cnc, GdaStatement *select_stmt, gbool
 		g_assert (GDA_SQL_ANY_PART (ust)->type == GDA_SQL_ANY_STMT_UPDATE);
 
 		ust->table = gda_sql_table_new (GDA_SQL_ANY_PART (ust));
-		ust->table->table_name = g_strdup ((gchar *) target->table_name);
-		ust->cond = gda_compute_unique_table_row_condition (stsel, 
-								    GDA_META_TABLE (target->validity_meta_object),
-								    require_pk, error);
+		ust->table->table_name = g_strdup (tmp);
+		ust->cond = gda_compute_unique_table_row_condition_with_cnc (cnc, stsel, 
+									     GDA_META_TABLE (target->validity_meta_object),
+									     require_pk, error);
 		if (!ust->cond) {
 			retval = FALSE;
-			goto cleanup;
+			*update_stmt = NULL;
+			update_stmt = NULL; /* don't try anymore to build UPDATE statement */
 		}
-		GDA_SQL_ANY_PART (ust->cond)->parent = GDA_SQL_ANY_PART (ust);
+		else
+			GDA_SQL_ANY_PART (ust->cond)->parent = GDA_SQL_ANY_PART (ust);
 	}
         
 	if (delete_stmt) {
@@ -958,16 +1017,22 @@ gda_compute_dml_statements (GdaConnection *cnc, GdaStatement *select_stmt, gbool
 		g_assert (GDA_SQL_ANY_PART (dst)->type == GDA_SQL_ANY_STMT_DELETE);
 
 		dst->table = gda_sql_table_new (GDA_SQL_ANY_PART (dst));
-		dst->table->table_name = g_strdup ((gchar *) target->table_name);
-		dst->cond = gda_compute_unique_table_row_condition (stsel, 
-								    GDA_META_TABLE (target->validity_meta_object),
-								    require_pk, error);
+		dst->table->table_name = g_strdup (tmp);
+		if (update_stmt && ust->cond)
+			dst->cond = gda_sql_expr_copy (ust->cond);
+		else
+			dst->cond = gda_compute_unique_table_row_condition_with_cnc (cnc, stsel, 
+									     GDA_META_TABLE (target->validity_meta_object),
+									     require_pk, error);
 		if (!dst->cond) {
 			retval = FALSE;
-			goto cleanup;
+			*delete_stmt = NULL;
+			delete_stmt = NULL; /* don't try anymore to build DELETE statement */
 		}
-		GDA_SQL_ANY_PART (dst->cond)->parent = GDA_SQL_ANY_PART (dst);
+		else
+			GDA_SQL_ANY_PART (dst->cond)->parent = GDA_SQL_ANY_PART (dst);
 	}
+	g_free (tmp);
 
 	GSList *expr_list;
 	gint colindex;
@@ -986,18 +1051,22 @@ gda_compute_dml_statements (GdaConnection *cnc, GdaStatement *select_stmt, gbool
 		if (g_hash_table_lookup (fields_hash, selfield->field_name))
 			continue;
 		g_hash_table_insert (fields_hash, selfield->field_name, GINT_TO_POINTER (1));
+
+		gchar *str;
+		str = gda_sql_identifier_quote (selfield->field_name, cnc, NULL, FALSE, FALSE);
 		if (insert_stmt) {
 			GdaSqlField *field;
 			field = gda_sql_field_new (GDA_SQL_ANY_PART (ist));
-			field->field_name = g_strdup (selfield->field_name);
+			field->field_name = g_strdup (str);
 			ist->fields_list = g_slist_append (ist->fields_list, field);
 		}
 		if (update_stmt) {
 			GdaSqlField *field;
 			field = gda_sql_field_new (GDA_SQL_ANY_PART (ust));
-			field->field_name = g_strdup (selfield->field_name);
+			field->field_name = g_strdup (str);
 			ust->fields_list = g_slist_append (ust->fields_list, field);
 		}
+		g_free (str);
 
 		/* parameter for the inserted value */
 		GdaSqlExpr *expr;
@@ -1034,10 +1103,11 @@ gda_compute_dml_statements (GdaConnection *cnc, GdaStatement *select_stmt, gbool
 			g_set_error (error, GDA_SQL_ERROR, GDA_SQL_STRUCTURE_CONTENTS_ERROR,
 				     "%s", _("Could not compute any field to insert into"));
 			retval = FALSE;
-			goto cleanup;
 		}
-		ist->values_list = g_slist_append (NULL, insert_values_list);
-		ret_insert = g_object_new (GDA_TYPE_STATEMENT, "structure", sql_ist, NULL);
+		else {
+			ist->values_list = g_slist_append (NULL, insert_values_list);
+			ret_insert = g_object_new (GDA_TYPE_STATEMENT, "structure", sql_ist, NULL);
+		}
 	}
 	if (update_stmt)
 		ret_update = g_object_new (GDA_TYPE_STATEMENT, "structure", sql_ust, NULL);
@@ -1104,7 +1174,8 @@ gda_compute_select_statement_from_update (GdaStatement *update_stmt, GError **er
 	target = gda_sql_select_target_new (GDA_SQL_ANY_PART (sst->from));
 	sst->from->targets = g_slist_prepend (NULL, target);
 	target->expr = gda_sql_expr_new (GDA_SQL_ANY_PART (target));
-	g_value_set_string ((target->expr->value = gda_value_new (G_TYPE_STRING)), ust->table->table_name);
+	g_value_set_string ((target->expr->value = gda_value_new (G_TYPE_STRING)),
+			    ust->table->table_name);
 
 	/* WHERE */
 	sst->where_cond = gda_sql_expr_copy (ust->cond);
@@ -1200,7 +1271,6 @@ gda_identifier_equal (const gchar *id1, const gchar *id2)
 }
 
 
-static char *copy_ident (const gchar *ident);
 static char *concat_ident (const gchar *prefix, const gchar *ident);
 
 static gchar *sql_start_words[] = {
@@ -1235,6 +1305,25 @@ static gchar *sql_middle_words[] = {
 	"SET"
 };
 
+static gchar *
+prepare_sql_identifier_for_compare (gchar *str)
+{
+	if (!str || (*str == '"'))
+		return str;
+	else {
+		gchar *ptr;
+		for (ptr = str; *ptr; ptr++)
+			*ptr = g_ascii_tolower (*ptr);
+		return str;
+	}
+}
+
+static gint
+cmp_func (gconstpointer a, gconstpointer b)
+{
+	return g_strcmp0 (*((gchar**) a), *((gchar**) b));
+}
+
 /**
  * gda_completion_list_get
  * @cnc: a #GdaConnection object
@@ -1258,6 +1347,8 @@ gda_completion_list_get (GdaConnection *cnc, const gchar *sql, gint start, gint
 	if (!cnc) 
 		return NULL;
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
+	if (!sql || !(*sql))
+		return NULL;
 	if (end < start)
 		return NULL;
 
@@ -1301,15 +1392,11 @@ gda_completion_list_get (GdaConnection *cnc, const gchar *sql, gint start, gint
 		else
 			goto compl_finished;
 	}
-	
-	if (*obj_name == '"') 
-		_remove_quotes (obj_name);
-	
-	if (obj_schema) {
-		if (*obj_schema == '"') 
-			_remove_quotes (obj_schema);
-		g_value_take_string ((schema_value = gda_value_new (G_TYPE_STRING)), obj_schema);
-	}
+		
+	prepare_sql_identifier_for_compare (obj_name);
+	if (obj_schema)
+		g_value_take_string ((schema_value = gda_value_new (G_TYPE_STRING)),
+				     prepare_sql_identifier_for_compare (obj_schema));
 	
 	/*
 	 * complete with "table" or "schema.table"
@@ -1340,7 +1427,7 @@ gda_completion_list_get (GdaConnection *cnc, const gchar *sql, gint start, gint
 					if (schema_value) 
 						str = concat_ident (obj_schema, tname);
 					else
-						str = copy_ident (tname);
+						str = g_strdup (tname);
 					g_array_append_val (compl, str);
 				}
 			}
@@ -1367,7 +1454,7 @@ gda_completion_list_get (GdaConnection *cnc, const gchar *sql, gint start, gint
 				cname = g_value_get_string (cvalue);
 				if (!strncmp (cname, obj_name, len)) {
 					gchar *str;
-					str = copy_ident (cname);
+					str = g_strdup (cname);
 					g_array_append_val (compl, str);
 				}
 			}
@@ -1394,7 +1481,7 @@ gda_completion_list_get (GdaConnection *cnc, const gchar *sql, gint start, gint
 				if (!strncmp (tname, obj_name, len)) {
 					char *str;
 					GdaDataModel *m2;
-					str = copy_ident (tname);
+					str = g_strdup (tname);
 				
 					m2 = gda_meta_store_extract (store, 
 								     "SELECT table_name FROM _tables WHERE table_schema = ##schema::string", 
@@ -1414,7 +1501,7 @@ gda_completion_list_get (GdaConnection *cnc, const gchar *sql, gint start, gint
 						
 						g_object_unref (m2);
 					}
-					free (str);
+					g_free (str);
 				}
 			}
 		}
@@ -1448,6 +1535,23 @@ gda_completion_list_get (GdaConnection *cnc, const gchar *sql, gint start, gint
 	g_free (text);
 	if (compl) {
 		if (compl->len >= 1) {
+			/* sort */
+			gint i;
+			g_array_sort (compl, cmp_func);
+
+			/* remove duplicates if any */
+			for (i = 1; i < compl->len; ) {
+				gchar *current, *before;
+				current = g_array_index (compl, gchar*, i);
+				before = g_array_index (compl, gchar*, i - 1);
+				if (!strcmp (current, before)) {
+					g_free (current);
+					g_array_remove_index (compl, i);
+				}
+				else
+					i++;
+			}
+
 			gchar **ptr;
 			ptr = (gchar**) compl->data;
 			g_array_free (compl, FALSE);
@@ -1462,26 +1566,6 @@ gda_completion_list_get (GdaConnection *cnc, const gchar *sql, gint start, gint
 		return NULL;
 }
 
-
-static char *
-copy_ident (const gchar *ident)
-{
-	char *str;
-	gint tlen = strlen (ident);
-	if (gda_sql_identifier_needs_quotes (ident)) {
-		str = malloc (sizeof (char) * (tlen + 3));
-		*str = '"';
-		strcpy (str+1, ident);
-		str [tlen + 1] = '"';
-		str [tlen + 2] = 0;
-	}
-	else {
-		str = malloc (sizeof (char) * (tlen + 1));
-		strcpy (str, ident);		
-	}
-	return str;
-}
-
 static char *
 concat_ident (const char *prefix, const gchar *ident)
 {
@@ -1492,31 +1576,14 @@ concat_ident (const char *prefix, const gchar *ident)
 	if (prefix)
 		plen = strlen (prefix) + 1;
 
-	if (gda_sql_identifier_needs_quotes (ident)) {
-		str = malloc (sizeof (char) * (plen + tlen + 3));
-		if (prefix) {
-			strcpy (str, prefix);
-			str [plen - 1] = '.';
-			str [plen] = '"';
-			strcpy (str + plen + 1, ident);
-		}
-		else {
-			*str = '"';
-			strcpy (str+1, ident);
-		}
-		str [plen + tlen + 1] = '"';
-		str [plen + tlen + 2] = 0;
-	}
-	else {
-		str = malloc (sizeof (char) * (plen + tlen + 1));
-		if (prefix) {
-			strcpy (str, prefix);
-			str [plen - 1] = '.';
-			strcpy (str + plen, ident);
-		}
-		else
-			strcpy (str, ident);
+	str = malloc (sizeof (char) * (plen + tlen + 1));
+	if (prefix) {
+		strcpy (str, prefix);
+		str [plen - 1] = '.';
+		strcpy (str + plen, ident);
 	}
+	else
+		strcpy (str, ident);
 	return str;
 }
 
@@ -1555,6 +1622,297 @@ gda_sql_identifier_split (const gchar *id)
 		return NULL;
 }
 
+static gboolean _sql_identifier_needs_quotes (const gchar *str);
+
+/**
+ * gda_sql_identifier_quote
+ * @id: an SQL identifier
+ * @cnc: a #GdaConnection object, or %NULL
+ * @prov: a #GdaServerProvider object, or %NULL
+ * @for_meta_store set to %TRUE if the returned string will be used in a #GdaMetaStore
+ * @force_quotes: set to %TRUE to force the returned string to be quoted
+ *
+ * Use this function for any SQL identifier to make sure that:
+ * <itemizedlist>
+ *   <listitem>
+ *     <para>it is correctly formatted
+ *           to be used with @cnc (if @cnc is %NULL, then some default SQL quoting rules will be applied,
+ *           similar to PostgreSQL's way) if @for_meta_store is %FALSE;
+ *     </para>
+ *   </listitem>
+ *   <listitem>
+ *     <para>it is correctly formatted to be used with the #GdaMetaStore's object associated to @cnc
+ *           is @for_meta_store is %TRUE.
+ *     </para>
+ *   </listitem>
+ * </itemizedlist>
+ *
+ * The @force_quotes allow some control of how to interpret @id: if %FALSE, then @id will be left
+ * unchanged most of the time (except for example if it's a reserved keyword), otherwise
+ * if @force_quotes is %TRUE, then the returned string will most probably have quotes around it
+ * to request that the database keep the case sensitiveness (but again, this may vary depending
+ * on the database being accessed through @cnc).
+ *
+ * For example, the following table gives the result of this function depending on the arguments
+ * when @cnc is %NULL (and @prov is also %NULL):
+ * <table frame="all">
+ *  <tgroup cols="6" colsep="1" rowsep="1" align="justify">
+ *    <thead>
+ *      <row>
+ *        <entry>id</entry>
+ *        <entry>for_meta_store=%FALSE, force_quotes=%FALSE</entry>
+ *        <entry>for_meta_store=%TRUE, force_quotes=%FALSE</entry>
+ *        <entry>for_meta_store=%FALSE, force_quotes=%TRUE</entry>
+ *        <entry>for_meta_store=%TRUE, force_quotes=%TRUE</entry>
+ *        <entry>remark</entry>
+ *      </row>
+ *    </thead>
+ *    <tbody>
+ *      <row>
+ *        <entry>"double word"</entry>
+ *        <entry>"double word"</entry>
+ *        <entry>"double word"</entry>
+ *        <entry>"double word"</entry>
+ *        <entry>"double word"</entry>
+ *        <entry>non allowed character in SQL identifier</entry>
+ *      </row>
+ *      <row>
+ *        <entry>"CapitalTest"</entry>
+ *        <entry>"CapitalTest"</entry>
+ *        <entry>"CapitalTest"</entry>
+ *        <entry>"CapitalTest"</entry>
+ *        <entry>"CapitalTest"</entry>
+ *        <entry>Mixed case SQL identifier, already quoted</entry>
+ *      </row>
+ *      <row>
+ *        <entry>CapitalTest</entry>
+ *        <entry>CapitalTest</entry>
+ *        <entry>capitaltest</entry>
+ *        <entry>"CapitalTest"</entry>
+ *        <entry>"CapitalTest"</entry>
+ *        <entry>Mixed case SQL identifier, non quoted</entry>
+ *      </row>
+ *      <row>
+ *        <entry>"mytable"</entry>
+ *        <entry>"mytable"</entry>
+ *        <entry>mytable</entry>
+ *        <entry>"mytable"</entry>
+ *        <entry>mytable</entry>
+ *        <entry>All lowser case, quoted</entry>
+ *      </row>
+ *      <row>
+ *        <entry>mytable</entry>
+ *        <entry>mytable</entry>
+ *        <entry>mytable</entry>
+ *        <entry>"mytable"</entry>
+ *        <entry>mytable</entry>
+ *        <entry>All lowser case</entry>
+ *      </row>
+ *      <row>
+ *        <entry>MYTABLE</entry>
+ *        <entry>MYTABLE</entry>
+ *        <entry>mytable</entry>
+ *        <entry>"MYTABLE"</entry>
+ *        <entry>"MYTABLE"</entry>
+ *        <entry>All upper case</entry>
+ *      </row>
+ *      <row>
+ *        <entry>"MYTABLE"</entry>
+ *        <entry>"MYTABLE"</entry>
+ *        <entry>"MYTABLE"</entry>
+ *        <entry>"MYTABLE"</entry>
+ *        <entry>"MYTABLE"</entry>
+ *        <entry>All upper case, quoted</entry>
+ *      </row>
+ *      <row>
+ *        <entry>desc</entry>
+ *        <entry>"desc"</entry>
+ *        <entry>"desc"</entry>
+ *        <entry>"desc"</entry>
+ *        <entry>"desc"</entry>
+ *        <entry>SQL reserved keyword</entry>
+ *      </row>
+ *      <row>
+ *        <entry>5ive</entry>
+ *        <entry>"5ive"</entry>
+ *        <entry>"5ive"</entry>
+ *        <entry>"5ive"</entry>
+ *        <entry>"5ive"</entry>
+ *        <entry>SQL identifier starting with a digit</entry>
+ *      </row>
+ *    </tbody>
+ *  </tgroup>
+ * </table>
+ *
+ * Here are a few examples of when and how to use this function:
+ * <itemizedlist>
+ *   <listitem>
+ *     <para>
+ *       When creating a table, the user has entered the table name, this function can be used to
+ *       create a valid SQL identifier from the user provided table name:
+ *       <programlisting>
+ * gchar *user_sqlid=...
+ * gchar *valid_sqlid = gda_sql_identifier_quote (user_sqlid, cnc, NULL, FALSE, FALSE);
+ * gchar *sql = g_strdup_printf ("CREATE TABLE %s ...", valid_sqlid);
+ * g_free (valid_sqlid);
+ *       </programlisting>
+ *       Note that this is an illustration and creating a table should be sone using a #GdaServerOperation
+ *       object.
+ *     </para>
+ *   </listitem>
+ *   <listitem>
+ *     <para>
+ *      When updating the meta data associated to a table which has been created with the code
+ *      above:
+ *      <programlisting>
+ * GValue table_name_value = { 0 };
+ * gchar* column_names[] = { (gchar*)"table_name" };
+ * GValue* column_values[] = { &table_name_value };
+ * GdaMetaContext mcontext = { (gchar*)"_tables", 1, column_names, column_values };
+ * g_value_init (&amp;table_name_value, G_TYPE_STRING);
+ * g_value_take_string (&amp;table_name_value, gda_sql_identifier_quote (user_sqlid, cnc, NULL, TRUE, FALSE);
+ * gda_connection_update_meta_store (cnc, &amp;mcontext, NULL);
+ * g_value_reset (&amp;table_name_value);
+ *       </programlisting>
+ *     </para>
+ *   </listitem>
+ *   <listitem>
+ *     <para>
+ *      When using a #GdaMetaStruct object to fetch information about a table (which has been created with
+ *      the code above):
+ *      <programlisting>
+ * GValue table_name_value = { 0 };
+ * g_value_init (&amp;table_name_value, G_TYPE_STRING);
+ * g_value_take_string (&amp;table_name_value, gda_sql_identifier_quote (user_sqlid, cnc, NULL, TRUE, FALSE);
+ * GdaMetaDbObject *dbo;
+ * dbo = gda_meta_struct_complement (mstruct, GDA_META_DB_TABLE, NULL, NULL, &amp;table_name_value, NULL);
+ * g_value_reset (&amp;table_name_value);
+ *       </programlisting>
+ *     </para>
+ *   </listitem>
+ * </itemizedlist>
+ *
+ *
+ * Note that @id must not be a composed SQL identifier (such as "mytable.mycolumn" which should be
+ * treated as the "mytable" and "mycolumn" SQL identifiers). If unsure, use gda_sql_identifier_split().
+ *
+ * Also note that if @cnc is %NULL, then it's possible to pass an non %NULL @prov to have a result specific
+ * to @prov.
+ *
+ * For more information, see the <link linkend="gen:sql_identifiers">SQL identifiers and abstraction</link> and
+ * <link linkend="information_schema:sql_identifiers">SQL identifiers in meta data</link> sections.
+ *
+ * Returns: the representation of @id ready to be used in SQL statement, as a new string,
+ *          or %NULL if @id is in a wrong format
+ *
+ * Since: 4.0.3
+ */
+gchar *
+gda_sql_identifier_quote (const gchar *id, GdaConnection *cnc, GdaServerProvider *prov,
+			  gboolean for_meta_store, gboolean force_quotes)
+{
+	g_return_val_if_fail (id && *id, NULL);
+	if (prov)
+		g_return_val_if_fail (GDA_IS_SERVER_PROVIDER (prov), NULL);
+	if (cnc) {
+		g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
+		if (prov)
+			g_return_val_if_fail (gda_connection_get_provider (cnc) == prov, NULL);
+		else
+			prov = gda_connection_get_provider (cnc);
+	}
+
+	if ((*id == '*') && (! id [1]))
+	    return g_strdup (id);
+
+	if (prov && PROV_CLASS (prov)->identifier_quote)
+		return PROV_CLASS (prov)->identifier_quote (prov, cnc, id,
+							    for_meta_store, force_quotes);
+
+	if (for_meta_store) {
+		gchar *tmp, *ptr;
+		tmp = _remove_quotes (g_strdup (id));
+		if (is_keyword (tmp)) {
+			ptr = gda_sql_identifier_add_quotes (tmp);
+			g_free (tmp);
+			return ptr;
+		}
+		else if (force_quotes) {
+			/* quote if non LC characters or digits at the 1st char or non allowed characters */
+			for (ptr = tmp; *ptr; ptr++) {
+				if (((*ptr >= 'a') && (*ptr <= 'z')) ||
+				    ((*ptr >= '0') && (*ptr <= '9') && (ptr != tmp)) ||
+				    (*ptr == '_'))
+					continue;
+				else {
+					ptr = gda_sql_identifier_add_quotes (tmp);
+					g_free (tmp);
+					return ptr;
+				}
+			}
+			return tmp;
+		}
+		else {
+			for (ptr = tmp; *ptr; ptr++) {
+				if (*id == '"') {
+					if (((*ptr >= 'a') && (*ptr <= 'z')) ||
+					    ((*ptr >= '0') && (*ptr <= '9') && (ptr != tmp)) ||
+					    (*ptr == '_'))
+						continue;
+					else {
+						ptr = gda_sql_identifier_add_quotes (tmp);
+						g_free (tmp);
+						return ptr;
+					}
+				}
+				else if ((*ptr >= 'A') && (*ptr <= 'Z'))
+					*ptr += 'a' - 'A';
+				else if ((*ptr >= '0') && (*ptr <= '9') && (ptr == tmp)) {
+					ptr = gda_sql_identifier_add_quotes (tmp);
+					g_free (tmp);
+					return ptr;
+				}
+			}
+			return tmp;
+		}
+	}
+	else {
+		/* default SQL standard */
+		if (*id == '"') {
+			/* there are already some quotes */
+			return g_strdup (id);
+		}
+		if (is_keyword (id) || _sql_identifier_needs_quotes (id) || force_quotes)
+			return gda_sql_identifier_add_quotes (id);
+		
+		/* nothing to do */
+		return g_strdup (id);
+	}
+}
+
+static gboolean
+_sql_identifier_needs_quotes (const gchar *str)
+{
+	const gchar *ptr;
+
+	g_return_val_if_fail (str, FALSE);
+	for (ptr = str; *ptr; ptr++) {
+		/* quote if 1st char is a number */
+		if ((*ptr <= '9') && (*ptr >= '0')) {
+			if (ptr == str)
+				return TRUE;
+			continue;
+		}
+		if (((*ptr >= 'A') && (*ptr <= 'Z')) ||
+		    ((*ptr >= 'a') && (*ptr <= 'z')))
+			continue;
+
+		if ((*ptr != '$') && (*ptr != '_') && (*ptr != '#'))
+			return TRUE;
+	}
+	return FALSE;
+}
+
 /*
  *  RFC 1738 defines that these characters should be escaped, as well
  *  any non-US-ASCII character or anything between 0x00 - 0x1F.
@@ -1908,11 +2266,6 @@ gda_parse_iso8601_time (GdaTime *timegda, const gchar *value)
 			ndigits++;
 		}
 
-		while (ndigits < 3) {
-			fraction *= 10;
-			ndigits++;
-		}
-
 		while (fraction > 0 && ndigits > 3) {
 			fraction /= 10;
 			ndigits--;
@@ -1968,11 +2321,6 @@ gda_parse_iso8601_timestamp (GdaTimestamp *timestamp, const gchar *value)
 			ndigits++;
 		}
 
-		while (ndigits < 3) {
-			fraction *= 10;
-			ndigits++;
-		}
-
 		while (fraction > 0 && ndigits > 3) {
 			fraction /= 10;
 			ndigits--;
