@@ -423,8 +423,10 @@ gda_blob_copy (gpointer boxed)
 	g_return_val_if_fail (src, NULL);
 
 	copy = g_new0 (GdaBlob, 1);
-	((GdaBinary *)copy)->data = g_memdup (((GdaBinary *)src)->data, ((GdaBinary *)src)->binary_length);
-	((GdaBinary *)copy)->binary_length = ((GdaBinary *)src)->binary_length;
+	if (((GdaBinary *)src)->data) {
+		((GdaBinary *)copy)->data = g_memdup (((GdaBinary *)src)->data, ((GdaBinary *)src)->binary_length);
+		((GdaBinary *)copy)->binary_length = ((GdaBinary *)src)->binary_length;
+	}
 	gda_blob_set_op (copy, src->op);
 	
 	return copy;
@@ -442,9 +444,11 @@ gda_blob_free (gpointer boxed)
 	GdaBlob *blob = (GdaBlob*) boxed;
 	
 	g_return_if_fail (blob);
-		
-	if (blob->op)
+
+	if (blob->op) {
 		g_object_unref (blob->op);
+		blob->op = NULL;
+	}
 	gda_binary_free ((GdaBinary *) blob);
 }
 
@@ -464,8 +468,7 @@ gda_blob_set_op (GdaBlob *blob, GdaBlobOp *op)
 	}
 	if (op) {
 		g_return_if_fail (GDA_IS_BLOB_OP (op));
-		blob->op = op;
-		g_object_ref (op);
+		blob->op = g_object_ref (op);
 	}
 }
 
@@ -1127,7 +1130,7 @@ gda_value_new_blob (const guchar *val, glong size)
 	GdaBinary *bin;
 
 	blob = g_new0 (GdaBlob, 1);
-	bin = (GdaBinary*)(&blob);
+	bin = (GdaBinary*)(blob);
 	bin->data = g_new (guchar, size);
         memcpy ((gpointer) bin->data, (gpointer) val, size);
         bin->binary_length = size;
@@ -1406,11 +1409,15 @@ void
 gda_value_set_binary (GValue *value, const GdaBinary *binary)
 {
 	g_return_if_fail (value);
-	g_return_if_fail (binary);
 	
 	l_g_value_unset (value);
 	g_value_init (value, GDA_TYPE_BINARY);
-	g_value_set_boxed (value, binary);
+	if (binary)
+		g_value_set_boxed (value, binary);
+	else {
+		GdaBinary bin = {NULL, 0};
+		g_value_set_boxed (value, &bin);
+	}
 }
 
 /**
@@ -2026,6 +2033,13 @@ gda_value_differ (const GValue *value1, const GValue *value2)
 		 * already have retruned */
 		return 1;
 
+	else if (g_type_is_a (type, G_TYPE_OBJECT)) {
+		if (g_value_get_object (value1) == g_value_get_object (value2))
+			return 0;
+		else
+			return -1;
+	}
+
 	g_warning ("%s() cannot handle values of type %s", __FUNCTION__, g_type_name (G_VALUE_TYPE (value1)));
 
 	return 1;
@@ -2306,6 +2320,13 @@ gda_value_compare (const GValue *value1, const GValue *value2)
 		return (t1 > t2) ? 1 : ((t1 == t2) ? 0: -1);
 	}
 
+	else if (g_type_is_a (type, G_TYPE_OBJECT)) {
+		if (g_value_get_object (value1) == g_value_get_object (value2))
+			return 0;
+		else
+			return -1;
+	}
+
 	g_warning ("%s() cannot handle values of type %s", __FUNCTION__, g_type_name (G_VALUE_TYPE (value1)));
 
 	return 0;
@@ -2489,6 +2510,13 @@ gda_ushort_get_type (void) {
  * where "xyz" is the octal representation of the byte, and the '\' (backslash) character
  * is converted to "\\".
  *
+ * Note that the backslash and newline characters are considered as printable characters and
+ * will not be represented by the "\xyz" representation.
+ *
+ * Use this function to get a representation as much readable by humans as possible of a binary
+ * chunk. Note that this function is internally called when transforming a binary value to
+ * a string for example when using g_value_transform() or gda_value_stringify().
+ *
  * Returns: a new string from @bin
  */
 gchar *
