@@ -1,6 +1,6 @@
 /* gda-handler-bin.c
  *
- * Copyright (C) 2005 - 2008 Vivien Malerba
+ * Copyright (C) 2005 - 2009 Vivien Malerba
  *
  * This Library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public License as
@@ -38,8 +38,6 @@ static GValue      *gda_handler_bin_get_value_from_sql     (GdaDataHandler *dh,
 							    GType type);
 static GValue      *gda_handler_bin_get_value_from_str     (GdaDataHandler *dh, const gchar *sql, 
 							    GType type);
-static guint        gda_handler_bin_get_nb_g_types       (GdaDataHandler *dh);
-static GType        gda_handler_bin_get_g_type_index     (GdaDataHandler *dh, guint index);
 static gboolean     gda_handler_bin_accepts_g_type       (GdaDataHandler * dh, GType type);
 
 static const gchar *gda_handler_bin_get_descr              (GdaDataHandler *dh);
@@ -59,6 +57,7 @@ gda_handler_bin_get_type (void)
 	static GType type = 0;
 
 	if (G_UNLIKELY (type == 0)) {
+		static GStaticMutex registering = G_STATIC_MUTEX_INIT;
 		static const GTypeInfo info = {
 			sizeof (GdaHandlerBinClass),
 			(GBaseInitFunc) NULL,
@@ -77,8 +76,12 @@ gda_handler_bin_get_type (void)
 			NULL
 		};
 
-		type = g_type_register_static (G_TYPE_OBJECT, "GdaHandlerBin", &info, 0);
-		g_type_add_interface_static (type, GDA_TYPE_DATA_HANDLER, &data_entry_info);
+		g_static_mutex_lock (&registering);
+		if (type == 0) {
+			type = g_type_register_static (G_TYPE_OBJECT, "GdaHandlerBin", &info, 0);
+			g_type_add_interface_static (type, GDA_TYPE_DATA_HANDLER, &data_entry_info);
+		}
+		g_static_mutex_unlock (&registering);
 	}
 	return type;
 }
@@ -91,9 +94,7 @@ gda_handler_bin_data_handler_init (GdaDataHandlerIface *iface)
 	iface->get_value_from_sql = gda_handler_bin_get_value_from_sql;
 	iface->get_value_from_str = gda_handler_bin_get_value_from_str;
 	iface->get_sane_init_value = NULL;
-	iface->get_nb_g_types = gda_handler_bin_get_nb_g_types;
 	iface->accepts_g_type = gda_handler_bin_accepts_g_type;
-	iface->get_g_type_index = gda_handler_bin_get_g_type_index;
 	iface->get_descr = gda_handler_bin_get_descr;
 }
 
@@ -119,7 +120,7 @@ gda_handler_bin_init (GdaHandlerBin * hdl)
 	hdl->priv->valid_g_types[0] = GDA_TYPE_BINARY;
 	hdl->priv->valid_g_types[1] = GDA_TYPE_BLOB;
 
-	g_object_set_data (G_OBJECT (hdl), "name", _("InternalBin"));
+	g_object_set_data (G_OBJECT (hdl), "name", "InternalBin");
 	g_object_set_data (G_OBJECT (hdl), "descr", _("Binary representation"));
 }
 
@@ -197,7 +198,7 @@ gda_handler_bin_get_sql_from_value (GdaDataHandler *iface, const GValue *value)
 		}
 	}
 	else
-		retval = g_strdup (NULL);
+		retval = g_strdup ("NULL");
 
 	return retval;
 }
@@ -265,35 +266,23 @@ gda_handler_bin_get_value_from_str (GdaDataHandler *iface, const gchar *str, GTy
 {
 	GdaHandlerBin *hdl;
 	GValue *value = NULL;
-	GdaBinary bin;
 
 	g_return_val_if_fail (iface && GDA_IS_HANDLER_BIN (iface), NULL);
 	hdl = GDA_HANDLER_BIN (iface);
 	g_return_val_if_fail (hdl->priv, NULL);
 
 	if (type == GDA_TYPE_BINARY) {
-		if (gda_string_to_binary (str, &bin)) {
-			value = gda_value_new_binary (bin.data, bin.binary_length);
-			g_free (bin.data);
+		GdaBinary *bin;
+		bin = gda_string_to_binary (str);
+		if (bin) {
+			value = gda_value_new (GDA_TYPE_BINARY);
+			gda_value_take_binary (value, bin);
 		}
 	}
 
 	return value;
 }
 
-static guint
-gda_handler_bin_get_nb_g_types (GdaDataHandler *iface)
-{
-	GdaHandlerBin *hdl;
-
-	g_return_val_if_fail (iface && GDA_IS_HANDLER_BIN (iface), 0);
-	hdl = GDA_HANDLER_BIN (iface);
-	g_return_val_if_fail (hdl->priv, 0);
-
-	return hdl->priv->nb_g_types;
-}
-
-
 static gboolean
 gda_handler_bin_accepts_g_type (GdaDataHandler *iface, GType type)
 {
@@ -315,19 +304,6 @@ gda_handler_bin_accepts_g_type (GdaDataHandler *iface, GType type)
 	return found;
 }
 
-static GType
-gda_handler_bin_get_g_type_index (GdaDataHandler *iface, guint index)
-{
-	GdaHandlerBin *hdl;
-
-	g_return_val_if_fail (iface && GDA_IS_HANDLER_BIN (iface), G_TYPE_INVALID);
-	hdl = GDA_HANDLER_BIN (iface);
-	g_return_val_if_fail (hdl->priv, G_TYPE_INVALID);
-	g_return_val_if_fail (index < hdl->priv->nb_g_types, G_TYPE_INVALID);
-
-	return hdl->priv->valid_g_types[index];
-}
-
 static const gchar *
 gda_handler_bin_get_descr (GdaDataHandler *iface)
 {
