@@ -185,7 +185,10 @@ gda_capi_provider_class_init (GdaCapiProviderClass *klass)
         provider_class->delete_savepoint = gda_capi_provider_delete_savepoint;
 
 	provider_class->create_parser = gda_capi_provider_create_parser;
-	provider_class->statement_to_sql = gda_capi_provider_statement_to_sql;
+	provider_class->statement_to_sql = NULL; /* don't use gda_capi_provider_statement_to_sql()
+						  * because it only calls gda_statement_to_sql_extended().
+						  * Set it to gda_capi_provider_statement_to_sql() if it does
+						  * not call calls gda_statement_to_sql_extended() */
 	provider_class->statement_prepare = gda_capi_provider_statement_prepare;
 	provider_class->statement_execute = gda_capi_provider_statement_execute;
 
@@ -504,12 +507,13 @@ gda_capi_provider_create_operation (GdaServerProvider *provider, GdaConnection *
 	dir = gda_gbr_get_file_path (GDA_DATA_DIR, LIBGDA_ABI_NAME, NULL);
         file = gda_server_provider_find_file (provider, dir, str);
 	g_free (dir);
-        g_free (str);
 
         if (! file) {
-                g_set_error (error, 0, 0, _("Missing spec. file '%s'"), file);
+                g_set_error (error, 0, 0, _("Missing spec. file '%s'"), str);
+		g_free (str);
                 return NULL;
         }
+        g_free (str);
 
         op = gda_server_operation_new (type, file);
         g_free (file);
@@ -542,12 +546,13 @@ gda_capi_provider_render_operation (GdaServerProvider *provider, GdaConnection *
 	dir = gda_gbr_get_file_path (GDA_DATA_DIR, LIBGDA_ABI_NAME, NULL);
         file = gda_server_provider_find_file (provider, dir, str);
 	g_free (dir);
-        g_free (str);
 
         if (! file) {
-                g_set_error (error, 0, 0, _("Missing spec. file '%s'"), file);
+                g_set_error (error, 0, 0, _("Missing spec. file '%s'"), str);
+		g_free (str);
                 return NULL;
         }
+        g_free (str);
         if (!gda_server_operation_is_valid (op, file, error)) {
                 g_free (file);
                 return NULL;
@@ -881,6 +886,9 @@ gda_capi_provider_create_parser (GdaServerProvider *provider, GdaConnection *cnc
  * The implementation show here simply calls gda_statement_to_sql_extended() but the rendering
  * can be specialized to the database's SQL dialect, see the implementation of gda_statement_to_sql_extended()
  * and SQLite's specialized rendering for more details
+ *
+ * NOTE: This implementation MUST NOT call gda_statement_to_sql_extended() if it is
+ *       the GdaServerProvider::statement_to_sql() virtual method's implementation
  */
 static gchar *
 gda_capi_provider_statement_to_sql (GdaServerProvider *provider, GdaConnection *cnc,
@@ -1011,10 +1019,6 @@ gda_capi_provider_statement_execute (GdaServerProvider *provider, GdaConnection
                 return NULL;
 	}
 
-	if (! (model_usage & GDA_STATEMENT_MODEL_RANDOM_ACCESS) &&
-            ! (model_usage & GDA_STATEMENT_MODEL_CURSOR_FORWARD))
-                model_usage |= GDA_STATEMENT_MODEL_RANDOM_ACCESS;
-	
         allow_noparam = (model_usage & GDA_STATEMENT_MODEL_ALLOW_NOPARAM) &&
                 (gda_statement_get_statement_type (stmt) == GDA_SQL_STATEMENT_SELECT);
 	
