@@ -13,27 +13,29 @@ main (int argc, char **argv)
 {
 	int number_failed = 0;
 
-	gda_init ("check-providers", PACKAGE_VERSION, argc, argv);
+	/* set up test environment */
+	g_setenv ("GDA_TOP_BUILD_DIR", TOP_BUILD_DIR, 0);
+	gda_init ();
 
-	setenv ("GDA_PROVIDERS_ROOT_DIR", GDA_PROVIDERS_ROOT_DIR, 0);
-	pinfo = gda_config_get_provider_by_name (PROVIDER);
+	pinfo = gda_config_get_provider_info (PROVIDER);
 	if (!pinfo) {
 		g_warning ("Could not find provider information for %s", PROVIDER);
-		return EXIT_FAILURE;
+		return EXIT_SUCCESS;
 	}
 	g_print ("Provider now tested: %s\n", pinfo->id);
 	
 	number_failed = prov_test_common_setup ();
 
 	if (cnc) {
-		number_failed += prov_test_common_create_tables_sql ();
-		number_failed += prov_test_common_check_schemas ();
+		number_failed += prov_test_common_check_meta ();
 		number_failed += prov_test_common_clean ();
 	}
 
 	if (! params_provided)
 		return EXIT_SUCCESS;
-	else
+	else {
+		g_print ("Test %s\n", (number_failed == 0) ? "Ok" : "failed");
 		return (number_failed == 0) ? EXIT_SUCCESS : EXIT_FAILURE;
+	}
 }
 
